@@ -25,33 +25,61 @@ class Pruner:
     Parameters
     -----------
     network : pandas df
-        kinase-site prediction network where there is an accession, site, kinase, and score column
+        weighted kinase-site prediction network where there is an accession, site, kinase, and score column
     logger : 
         logger used for pruning
     phospho_type : str
         phospho_type(s) to use when building pruned networks
-    columns : dict
-        relevant columns in network
+    acc_col : str
+        the name of the column containing Uniprot Accession IDs for each substrate in the weighted network
+    site_col : str
+        the name of the column containing the residue type and location of each substrate in the weighted network (Y1268, S44, etc.)
+    nonweight_cols : list
+        indicates the non-weight containing columns in the network (these will be removed in the final processed network, as they are not needed). If None, will automatically look
+            for any non-numeric columns and removes them.
     """
-    def __init__(self, network, logger, phospho_type = 'Y',):
+    def __init__(self, network, logger, phospho_type = 'Y',acc_col = 'substrate_acc', site_col = 'site', nonweight_cols = ['substrate_acc','site','substrate_id', 'substrate_name', 'pep']):
         
 
-        self.phospho_type = tuple(phospho_type)
+        self.phospho_type = phospho_type
         self.logger = logger
-        self.network = network[network['site'].str.startswith(self.phospho_type)]
+        self.network = network[network['site'].str.startswith(tuple(self.phospho_type))]
         
-
+        #load human reference phosphoproteome, obtain the correct phosphosites (Y or ST), and combine duplicate entries
         self.compendia = config.HUMAN_REF_COMPENDIA
-        self.compendia = self.compendia[self.compendia[config.KSTAR_SITE].str.startswith(self.phospho_type)]
+        self.compendia = self.compendia[self.compendia[config.KSTAR_SITE].str.startswith(tuple(self.phospho_type))]
         self.compendia = self.compendia[[config.KSTAR_ACCESSION, config.KSTAR_SITE, 'KSTAR_NUM_COMPENDIA']]
         self.compendia = self.compendia.groupby([config.KSTAR_ACCESSION, config.KSTAR_SITE]).max().reset_index()
-
-        self.network = pd.merge(self.network, self.compendia, how = 'inner', left_on = ['substrate_acc', 'site'], right_on=[config.KSTAR_ACCESSION, config.KSTAR_SITE] )
-        self.network = self.network.drop(columns=['substrate_acc','site','substrate_id', 'substrate_name', 'pep'])
+        #merge the human phosphoproteome compendia, which contains study bias information for all phosphosites, with the network
+        self.network = pd.merge(self.network, self.compendia, how = 'inner', left_on = [acc_col, site_col], right_on=[config.KSTAR_ACCESSION, config.KSTAR_SITE] )
+        
+        #update index to indicate site associated with each row
         self.network = self.network.set_index([config.KSTAR_ACCESSION, config.KSTAR_SITE])
+        #drop unneeded columns (or maybe drop any non number columns)
+        if nonweight_cols is not None:
+            nonweight_cols = [col for col in nonweight_cols if col in self.network.columns]
+            self.network = self.network.drop(columns=nonweight_cols)
+            
+        #check to make sure all remaining columns contain only numeric values
+        non_numeric_cols = self.network.select_dtypes(exclude = np.number).columns
+        if len(non_numeric_cols) > 0:
+            print('Following non-numeric columns were identified:', ', '.join(non_numeric_cols))
+            print('It is assumed that these columns do not contain kinase-substrate weights and were removed. If this incorrect, please make sure weight containing columns only contain numeric values')
+            self.network = self.network.select_dtypes(include = np.number)
 
         self.network = self.network.replace('-',np.nan)
         self.network = self.network.dropna(axis='columns', how ='all')
+        
+        #check for duplicate entries/indices
+        if len(self.network.index.duplicated()) > 0:
+            tmp = self.network[self.network.index.duplicated(keep = False)]
+            #check to see if duplicate entries contain different information. If they do not, drop one row. If they do, return error.
+            if any([i and v for i,v in zip(self.network.index.duplicated(keep = False), ~self.network.duplicated(keep = False).values)]):
+                print('Duplicate entries found in network that contain conflicting weights, please fix before running pruning')
+                exit()
+            else:
+                print('Duplicate entries found in network, removing repeat rows')
+                self.network = self.network[~self.network.index.duplicated()]
 
         self.kinases = list(self.network.columns)
         self.kinases.remove('KSTAR_NUM_COMPENDIA')
@@ -93,7 +121,7 @@ class Pruner:
                 pruned_network.at[sample.name, kinase] = 1
                 site_sizes[sample.name] = site_sizes[sample.name] + 1
                 if site_sizes[sample.name] >= site_limit:
-                    network.at[sample.name,:] = np.nan 
+                    network.loc[sample.name,:] = np.nan 
         
         pruned_network = pruned_network.reset_index().melt(id_vars=[config.KSTAR_ACCESSION, config.KSTAR_SITE]).dropna()
         pruned_network = pruned_network.rename(columns={'variable':config.KSTAR_KINASE})
@@ -127,7 +155,7 @@ class Pruner:
         pruned_network = pd.concat(pruned_network)
 
         pruned_network.to_csv(f"{name}.tsv", sep = "\t", index=False)
-        # return pruned_network
+
         
     def calculate_compendia_sizes(self, kinase_size):
         """
@@ -179,6 +207,12 @@ class Pruner:
             key : <network_id>_<i>
             value : pruned network
         """
+        self.kinase_size = kinase_size
+        self.site_limit = site_limit
+        self.num_networks = num_networks
+        self.network_id = network_id
+        self.network_dir = odir
+        self.use_compendia = True
         
         compendia_sizes = self.calculate_compendia_sizes(kinase_size)
         
@@ -194,7 +228,6 @@ class Pruner:
             pool = multiprocessing.Pool(processes = PROCESSES)
             comp_sizes = itertools.repeat(compendia_sizes, num_networks)
             limits = itertools.repeat(site_limit, num_networks)
-            # names = [os.path.join(odir,f"{network_id}_{i}") for i in range(num_networks)] 
             odirs = itertools.repeat(odir, num_networks)
             iterable = zip(comp_sizes, limits, odirs)
 
@@ -204,42 +237,92 @@ class Pruner:
         else:
             pruned_networks = []
             for i in range(num_networks):
-                self.compendia_pruned_network(compendia_sizes, site_limit, os.path.join(odir,f"{network_id}_{i}") )
+                self.compendia_pruned_network(compendia_sizes, site_limit, odir )
+        
+  
+
+    def build_multiple_networks(self, kinase_size, site_limit, num_networks, network_id, odir, PROCESSES = 1):
+        """
+        Basic Network Generation - only takes into account score when determining sites a kinase 
+        connects to
+        """
+        self.kinase_size = kinase_size
+        self.site_limit = site_limit
+        self.num_networks = num_networks
+        self.network_id = network_id
+        self.network_dir = odir
+        self.use_compendia = False
+        
+        odir = os.path.join(odir, "work")
+        # MULTIPROCESSING
+        if PROCESSES > 1:
+            pool = multiprocessing.Pool(processes = PROCESSES)
+            network_iter = itertools.repeat(self.network, num_networks)
+            size_iter = itertools.repeat(kinase_size, num_networks)
+            limit_iter = itertools.repeat(site_limit, num_networks)
+            iterable = zip(network_iter, size_iter, limit_iter)
+            pruned_networks = pool.starmap(self.build_pruned_network, iterable)
+            #save each network
+            for i in range(len(pruned_networks)):
+                net = pruned_networks[i]
+                name = os.path.join(odir,''.join(random.choices(string.ascii_uppercase + string.digits, k=10)))
+                net.to_csv(f"{name}.tsv", sep = "\t", index=False) 
+        # SINGLE CORE PROCESSING
+        else:
+            pruned_networks = []
+            for i in range(num_networks):
+                name = os.path.join(odir,''.join(random.choices(string.ascii_uppercase + string.digits, k=10)))
+                net = self.build_pruned_network(self.network, kinase_size, site_limit)
+                net.to_csv(f"{name}.tsv", sep = "\t", index=False)
                 
+    def save_networks(self):
+        """
+        Save the pruned networks generated by the 'build_multiple_networks' or 'build_multiple_compendia_networks' as a pickle to be loaded by KSTAR
+        
+        """
+        self.logger.info("Saving pruning results")
+
+        
+        if self.use_compendia:
+            suffix = f"{self.phospho_type}_compendia_{self.kinase_size}_limit_{self.site_limit}"
+        else:
+            suffix = f"{self.phospho_type}_{self.kinase_size}_limit_{self.site_limit}"
+
+        # TODO: rename work directory and rename files
+        os.rename(os.path.join(self.network_dir, "work"), os.path.join(self.network_dir, "INDIVIDUAL_NETWORKS"))
+        odir_ind = os.path.join(self.network_dir, "INDIVIDUAL_NETWORKS")
+        # rename files and make network map
+        temp_filenames = os.listdir(odir_ind)
+        network_map = {}
+        for i in range(len(temp_filenames)):
+            network_map[f"{self.network_id}_{i}"] = pd.read_table(os.path.join(odir_ind,temp_filenames[i]))
+            os.rename(os.path.join(odir_ind, temp_filenames[i]), os.path.join(odir_ind, f"{self.network_id}_{i}_{suffix}.tsv"))
+
+        pickle.dump( network_map, open( f"{self.network_dir}/{self.network_id}_{suffix}.p", "wb" ) )
         
-        # pruned_dict = {}
-        # for i in range(num_networks):
-        #     name = os.path.join(odir,f"{network_id}_{i}")
-        #     pruned_dict[f"{network_id}_{i}"] = pd.read_table(f"{name}.tsv")
         
 
-        # return pruned_dict
-
-    # def build_multiple_networks(self, kinase_size, site_limit, num_networks, network_id):
-    #     """
-    #     Basic Network Generation - only takes into account score when determining sites a kinase 
-    #     connects to
-    #     """
-    #     # MULTIPROCESSING
-    #     if config.PROCESSES > 1:
-    #         pool = multiprocessing.Pool(processes = config.PROCESSES)
-    #         network_iter = itertools.repeat(self.network, num_networks)
-    #         size_iter = itertools.repeat(kinase_size, num_networks)
-    #         limit_iter = itertools.repeat(site_limit, num_networks)
-    #         iterable = zip(network_iter, size_iter, limit_iter)
-    #         pruned_networks = pool.starmap(self.build_pruned_network, iterable)
         
-    #     # SINGLE CORE PROCESSING
-    #     else:
-    #         pruned_networks = []
-    #         for i in range(num_networks):
-    #             net = self.build_pruned_network(self.network, kinase_size, site_limit)
-    #             pruned_networks.append(net)
-
-    #     pruned_dict = {}
-    #     for i in range(len(pruned_networks)):
-    #         pruned_dict[f"{network_id}_{i}"] = pruned_networks[i]
-    #     return pruned_dict
+    def save_run_information(self):
+        """
+        Save information about the generation of networks during run_pruning, 
+        including the parameters used for generation. Primarily used when running bash script.
+        
+        """
+        with open(f"{self.network_dir}/RUN_INFORMATION.txt", "w") as info_file:
+            info_file.write("*************************************************\n")
+            info_file.write(f"Pruning Information for {self.network_id}\n")
+            info_file.write("*************************************************\n")
+            info_file.write(f"Date Run\t\t{datetime.now()}\n")
+            info_file.write(f"Phospho Type\t{self.phospho_type}\n")
+            info_file.write(f"Kinase Size\t\t{self.kinase_size}\n")
+            info_file.write(f"Site Limit\t\t{self.site_limit}\n")
+            info_file.write(f"# of Networks\t{self.num_networks}\n")
+            info_file.write(f"Use Compendia\t{self.use_compendia}\n")
+            if self.use_compendia:
+                compendia_sizes = pruner.calculate_compendia_sizes(self.kinase_size)
+                for comp, size in compendia_sizes.items():
+                    info_file.write(f"\tCompendia {comp}\t{size}\n")
 
 
 """
@@ -304,7 +387,8 @@ def process_args(results):
     return network, log, use_compendia
 
 
-def run_pruning(network, log, use_compendia, phospho_type, kinase_size, site_limit, num_networks, network_id, odir, PROCESSES = 1):
+def run_pruning(network, log, phospho_type, kinase_size, site_limit, num_networks, network_id, odir, use_compendia = True,
+                acc_col = 'substrate_acc', site_col = 'site', netcols_todrop = ['substrate_acc','site','substrate_id', 'substrate_name', 'pep'], PROCESSES = 1):
     """
     Generate pruned networks from a weighted kinase-substrate graph and log run information
     
@@ -335,13 +419,13 @@ def run_pruning(network, log, use_compendia, phospho_type, kinase_size, site_lim
         prune object that contains the number of pruned networks indicated by the num_networks paramater
     """
     log.info("Running pruning algorithm")
-    pruner = Pruner(network, log, phospho_type)
+    pruner = Pruner(network, log, phospho_type, acc_col = acc_col, site_col = site_col, nonweight_cols = netcols_todrop)
     if use_compendia:
         log.info("Pruning using compendia ratios")
         pruner.build_multiple_compendia_networks(kinase_size, site_limit, num_networks, network_id, odir, PROCESSES = PROCESSES)
-    # else:
-    #     log.info("Pruning without using compendia")
-    #     pruned_networks = pruner.build_multiple_networks(kinase_size, site_limit, num_networks, network_id)
+    else:
+        log.info("Pruning without using compendia")
+        pruned_networks = pruner.build_multiple_networks(kinase_size, site_limit, num_networks, network_id, odir, PROCESSES = PROCESSES)
     return pruner
 
 def save_pruning(phospho_type, network_id, kinase_size, site_limit, use_compendia, odir, log):
